/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#pragma once

#include <sstream>
#include "boost/any.hpp"
#include <Specific.hh>
#include <Encoder.hh>
#include <Decoder.hh>



namespace HK101 {

static const int PDM_PIXEL_RATE = 64;
static const int NUM_PDM = 37;
static const int SPARE1_LEN = 8;
static const int SPARE3_LEN = 4;


struct PDMBlock {
    bool pdmVal;
    int32_t spare_2;
    int32_t pdmID;
    int32_t sipmTemp1;
    int32_t sipmTemp2;
    int32_t sipmTemp3;
    int32_t citrocBoardTemp;
    int32_t citrocBoardVoltage1;
    int32_t citrocBoardVoltage2;
    int32_t citrocBoardCurrent1;
    int32_t citrocBoardCurrent2;
    int32_t citrocBoardCurrent3;
    int32_t fpgaTemperature;
    int32_t fpgaVoltage;
    int32_t sipmHighVoltage;
    int32_t sipmCurrent;
    int32_t cameraRate;
    int32_t pdmTriggerRate;
    std::vector<int32_t > pdmPixelsRate;
    std::vector<int32_t > spare_3;
    PDMBlock() :
        pdmVal(bool()),
        spare_2(int32_t()),
        pdmID(int32_t()),
        sipmTemp1(int32_t()),
        sipmTemp2(int32_t()),
        sipmTemp3(int32_t()),
        citrocBoardTemp(int32_t()),
        citrocBoardVoltage1(int32_t()),
        citrocBoardVoltage2(int32_t()),
        citrocBoardCurrent1(int32_t()),
        citrocBoardCurrent2(int32_t()),
        citrocBoardCurrent3(int32_t()),
        fpgaTemperature(int32_t()),
        fpgaVoltage(int32_t()),
        sipmHighVoltage(int32_t()),
        sipmCurrent(int32_t()),
        cameraRate(int32_t()),
        pdmTriggerRate(int32_t()),
        pdmPixelsRate(std::vector<int32_t >()),
        spare_3(std::vector<int32_t >())
        { }
};

struct HK101 {
    int32_t telescopeID;
    int32_t type;
    int32_t subType;
    int32_t ssc;
    int32_t packetLength;
    int32_t year;
    int32_t month;
    int32_t day;
    int32_t hours;
    int32_t minutes;
    int32_t seconds;
    bool validTime;
    int32_t timeTagNanosec;
    int32_t eventCounter;
    bool lid;
    bool fibSt;
    bool fibCont;
    bool fibPuls;
    int32_t rgbCont;
    int32_t rgbPuls;
    int32_t spare_0;
    int32_t cameraTemperature1;
    int32_t cameraTemperature2;
    int32_t cameraTemperature3;
    int32_t cameraTemperature4;
    int32_t cameraHumidity1;
    int32_t cameraHumidity2;
    int32_t cameraHumidity3;
    int32_t cameraHumidity4;
    int32_t bluLaserFrequency;
    int32_t bluLaserDuration;
    int32_t bluLaserVoltage;
    int32_t greenLaserFrequency;
    int32_t greenLaserDuration;
    int32_t greenLaserVoltage;
    int32_t laserDiodeTemperature;
    int32_t cameraRateMeter;
    int32_t totalCameraRateMeter;
    int32_t deadTime;
    std::vector<int32_t > spare_1;
    std::vector<PDMBlock > PDMs;
    HK101() :
        telescopeID(int32_t()),
        type(int32_t()),
        subType(int32_t()),
        ssc(int32_t()),
        packetLength(int32_t()),
        year(int32_t()),
        month(int32_t()),
        day(int32_t()),
        hours(int32_t()),
        minutes(int32_t()),
        seconds(int32_t()),
        validTime(bool()),
        timeTagNanosec(int32_t()),
        eventCounter(int32_t()),
        lid(bool()),
        fibSt(bool()),
        fibCont(bool()),
        fibPuls(bool()),
        rgbCont(int32_t()),
        rgbPuls(int32_t()),
        spare_0(int32_t()),
        cameraTemperature1(int32_t()),
        cameraTemperature2(int32_t()),
        cameraTemperature3(int32_t()),
        cameraTemperature4(int32_t()),
        cameraHumidity1(int32_t()),
        cameraHumidity2(int32_t()),
        cameraHumidity3(int32_t()),
        cameraHumidity4(int32_t()),
        bluLaserFrequency(int32_t()),
        bluLaserDuration(int32_t()),
        bluLaserVoltage(int32_t()),
        greenLaserFrequency(int32_t()),
        greenLaserDuration(int32_t()),
        greenLaserVoltage(int32_t()),
        laserDiodeTemperature(int32_t()),
        cameraRateMeter(int32_t()),
        totalCameraRateMeter(int32_t()),
        deadTime(int32_t()),
        spare_1(std::vector<int32_t >()),
        PDMs(std::vector<PDMBlock >())
        { }
};

}
namespace avro {
template<> struct codec_traits<HK101::PDMBlock> {
    static void encode(Encoder& e, const HK101::PDMBlock& v) {
        avro::encode(e, v.pdmVal);
        avro::encode(e, v.spare_2);
        avro::encode(e, v.pdmID);
        avro::encode(e, v.sipmTemp1);
        avro::encode(e, v.sipmTemp2);
        avro::encode(e, v.sipmTemp3);
        avro::encode(e, v.citrocBoardTemp);
        avro::encode(e, v.citrocBoardVoltage1);
        avro::encode(e, v.citrocBoardVoltage2);
        avro::encode(e, v.citrocBoardCurrent1);
        avro::encode(e, v.citrocBoardCurrent2);
        avro::encode(e, v.citrocBoardCurrent3);
        avro::encode(e, v.fpgaTemperature);
        avro::encode(e, v.fpgaVoltage);
        avro::encode(e, v.sipmHighVoltage);
        avro::encode(e, v.sipmCurrent);
        avro::encode(e, v.cameraRate);
        avro::encode(e, v.pdmTriggerRate);
        avro::encode(e, v.pdmPixelsRate);
        avro::encode(e, v.spare_3);
    }
    static void decode(Decoder& d, HK101::PDMBlock& v) {
        if (avro::ResolvingDecoder *rd =
            dynamic_cast<avro::ResolvingDecoder *>(&d)) {
            const std::vector<size_t> fo = rd->fieldOrder();
            for (std::vector<size_t>::const_iterator it = fo.begin();
                it != fo.end(); ++it) {
                switch (*it) {
                case 0:
                    avro::decode(d, v.pdmVal);
                    break;
                case 1:
                    avro::decode(d, v.spare_2);
                    break;
                case 2:
                    avro::decode(d, v.pdmID);
                    break;
                case 3:
                    avro::decode(d, v.sipmTemp1);
                    break;
                case 4:
                    avro::decode(d, v.sipmTemp2);
                    break;
                case 5:
                    avro::decode(d, v.sipmTemp3);
                    break;
                case 6:
                    avro::decode(d, v.citrocBoardTemp);
                    break;
                case 7:
                    avro::decode(d, v.citrocBoardVoltage1);
                    break;
                case 8:
                    avro::decode(d, v.citrocBoardVoltage2);
                    break;
                case 9:
                    avro::decode(d, v.citrocBoardCurrent1);
                    break;
                case 10:
                    avro::decode(d, v.citrocBoardCurrent2);
                    break;
                case 11:
                    avro::decode(d, v.citrocBoardCurrent3);
                    break;
                case 12:
                    avro::decode(d, v.fpgaTemperature);
                    break;
                case 13:
                    avro::decode(d, v.fpgaVoltage);
                    break;
                case 14:
                    avro::decode(d, v.sipmHighVoltage);
                    break;
                case 15:
                    avro::decode(d, v.sipmCurrent);
                    break;
                case 16:
                    avro::decode(d, v.cameraRate);
                    break;
                case 17:
                    avro::decode(d, v.pdmTriggerRate);
                    break;
                case 18:
                    avro::decode(d, v.pdmPixelsRate);
                    break;
                case 19:
                    avro::decode(d, v.spare_3);
                    break;
                default:
                    break;
                }
            }
        } else {
            avro::decode(d, v.pdmVal);
            avro::decode(d, v.spare_2);
            avro::decode(d, v.pdmID);
            avro::decode(d, v.sipmTemp1);
            avro::decode(d, v.sipmTemp2);
            avro::decode(d, v.sipmTemp3);
            avro::decode(d, v.citrocBoardTemp);
            avro::decode(d, v.citrocBoardVoltage1);
            avro::decode(d, v.citrocBoardVoltage2);
            avro::decode(d, v.citrocBoardCurrent1);
            avro::decode(d, v.citrocBoardCurrent2);
            avro::decode(d, v.citrocBoardCurrent3);
            avro::decode(d, v.fpgaTemperature);
            avro::decode(d, v.fpgaVoltage);
            avro::decode(d, v.sipmHighVoltage);
            avro::decode(d, v.sipmCurrent);
            avro::decode(d, v.cameraRate);
            avro::decode(d, v.pdmTriggerRate);
            avro::decode(d, v.pdmPixelsRate);
            avro::decode(d, v.spare_3);
        }
    }
};

template<> struct codec_traits<HK101::HK101> {
    static void encode(Encoder& e, const HK101::HK101& v) {
        avro::encode(e, v.telescopeID);
        avro::encode(e, v.type);
        avro::encode(e, v.subType);
        avro::encode(e, v.ssc);
        avro::encode(e, v.packetLength);
        avro::encode(e, v.year);
        avro::encode(e, v.month);
        avro::encode(e, v.day);
        avro::encode(e, v.hours);
        avro::encode(e, v.minutes);
        avro::encode(e, v.seconds);
        avro::encode(e, v.validTime);
        avro::encode(e, v.timeTagNanosec);
        avro::encode(e, v.eventCounter);
        avro::encode(e, v.lid);
        avro::encode(e, v.fibSt);
        avro::encode(e, v.fibCont);
        avro::encode(e, v.fibPuls);
        avro::encode(e, v.rgbCont);
        avro::encode(e, v.rgbPuls);
        avro::encode(e, v.spare_0);
        avro::encode(e, v.cameraTemperature1);
        avro::encode(e, v.cameraTemperature2);
        avro::encode(e, v.cameraTemperature3);
        avro::encode(e, v.cameraTemperature4);
        avro::encode(e, v.cameraHumidity1);
        avro::encode(e, v.cameraHumidity2);
        avro::encode(e, v.cameraHumidity3);
        avro::encode(e, v.cameraHumidity4);
        avro::encode(e, v.bluLaserFrequency);
        avro::encode(e, v.bluLaserDuration);
        avro::encode(e, v.bluLaserVoltage);
        avro::encode(e, v.greenLaserFrequency);
        avro::encode(e, v.greenLaserDuration);
        avro::encode(e, v.greenLaserVoltage);
        avro::encode(e, v.laserDiodeTemperature);
        avro::encode(e, v.cameraRateMeter);
        avro::encode(e, v.totalCameraRateMeter);
        avro::encode(e, v.deadTime);
        avro::encode(e, v.spare_1);
        avro::encode(e, v.PDMs);
    }
    static void decode(Decoder& d, HK101::HK101& v) {
        if (avro::ResolvingDecoder *rd =
            dynamic_cast<avro::ResolvingDecoder *>(&d)) {
            const std::vector<size_t> fo = rd->fieldOrder();
            for (std::vector<size_t>::const_iterator it = fo.begin();
                it != fo.end(); ++it) {
                switch (*it) {
                case 0:
                    avro::decode(d, v.telescopeID);
                    break;
                case 1:
                    avro::decode(d, v.type);
                    break;
                case 2:
                    avro::decode(d, v.subType);
                    break;
                case 3:
                    avro::decode(d, v.ssc);
                    break;
                case 4:
                    avro::decode(d, v.packetLength);
                    break;
                case 5:
                    avro::decode(d, v.year);
                    break;
                case 6:
                    avro::decode(d, v.month);
                    break;
                case 7:
                    avro::decode(d, v.day);
                    break;
                case 8:
                    avro::decode(d, v.hours);
                    break;
                case 9:
                    avro::decode(d, v.minutes);
                    break;
                case 10:
                    avro::decode(d, v.seconds);
                    break;
                case 11:
                    avro::decode(d, v.validTime);
                    break;
                case 12:
                    avro::decode(d, v.timeTagNanosec);
                    break;
                case 13:
                    avro::decode(d, v.eventCounter);
                    break;
                case 14:
                    avro::decode(d, v.lid);
                    break;
                case 15:
                    avro::decode(d, v.fibSt);
                    break;
                case 16:
                    avro::decode(d, v.fibCont);
                    break;
                case 17:
                    avro::decode(d, v.fibPuls);
                    break;
                case 18:
                    avro::decode(d, v.rgbCont);
                    break;
                case 19:
                    avro::decode(d, v.rgbPuls);
                    break;
                case 20:
                    avro::decode(d, v.spare_0);
                    break;
                case 21:
                    avro::decode(d, v.cameraTemperature1);
                    break;
                case 22:
                    avro::decode(d, v.cameraTemperature2);
                    break;
                case 23:
                    avro::decode(d, v.cameraTemperature3);
                    break;
                case 24:
                    avro::decode(d, v.cameraTemperature4);
                    break;
                case 25:
                    avro::decode(d, v.cameraHumidity1);
                    break;
                case 26:
                    avro::decode(d, v.cameraHumidity2);
                    break;
                case 27:
                    avro::decode(d, v.cameraHumidity3);
                    break;
                case 28:
                    avro::decode(d, v.cameraHumidity4);
                    break;
                case 29:
                    avro::decode(d, v.bluLaserFrequency);
                    break;
                case 30:
                    avro::decode(d, v.bluLaserDuration);
                    break;
                case 31:
                    avro::decode(d, v.bluLaserVoltage);
                    break;
                case 32:
                    avro::decode(d, v.greenLaserFrequency);
                    break;
                case 33:
                    avro::decode(d, v.greenLaserDuration);
                    break;
                case 34:
                    avro::decode(d, v.greenLaserVoltage);
                    break;
                case 35:
                    avro::decode(d, v.laserDiodeTemperature);
                    break;
                case 36:
                    avro::decode(d, v.cameraRateMeter);
                    break;
                case 37:
                    avro::decode(d, v.totalCameraRateMeter);
                    break;
                case 38:
                    avro::decode(d, v.deadTime);
                    break;
                case 39:
                    avro::decode(d, v.spare_1);
                    break;
                case 40:
                    avro::decode(d, v.PDMs);
                    break;
                default:
                    break;
                }
            }
        } else {
            avro::decode(d, v.telescopeID);
            avro::decode(d, v.type);
            avro::decode(d, v.subType);
            avro::decode(d, v.ssc);
            avro::decode(d, v.packetLength);
            avro::decode(d, v.year);
            avro::decode(d, v.month);
            avro::decode(d, v.day);
            avro::decode(d, v.hours);
            avro::decode(d, v.minutes);
            avro::decode(d, v.seconds);
            avro::decode(d, v.validTime);
            avro::decode(d, v.timeTagNanosec);
            avro::decode(d, v.eventCounter);
            avro::decode(d, v.lid);
            avro::decode(d, v.fibSt);
            avro::decode(d, v.fibCont);
            avro::decode(d, v.fibPuls);
            avro::decode(d, v.rgbCont);
            avro::decode(d, v.rgbPuls);
            avro::decode(d, v.spare_0);
            avro::decode(d, v.cameraTemperature1);
            avro::decode(d, v.cameraTemperature2);
            avro::decode(d, v.cameraTemperature3);
            avro::decode(d, v.cameraTemperature4);
            avro::decode(d, v.cameraHumidity1);
            avro::decode(d, v.cameraHumidity2);
            avro::decode(d, v.cameraHumidity3);
            avro::decode(d, v.cameraHumidity4);
            avro::decode(d, v.bluLaserFrequency);
            avro::decode(d, v.bluLaserDuration);
            avro::decode(d, v.bluLaserVoltage);
            avro::decode(d, v.greenLaserFrequency);
            avro::decode(d, v.greenLaserDuration);
            avro::decode(d, v.greenLaserVoltage);
            avro::decode(d, v.laserDiodeTemperature);
            avro::decode(d, v.cameraRateMeter);
            avro::decode(d, v.totalCameraRateMeter);
            avro::decode(d, v.deadTime);
            avro::decode(d, v.spare_1);
            avro::decode(d, v.PDMs);
        }
    }
};


}
