#include <stdexcept>
#include <iostream>
#include <csignal>
#include <fstream>
#include <vector>
#include <numeric>
#include <iostream>
#include <chrono>
#include <math.h>

#include <AvroPackets/c11.h>
#include <AvroPackets/c14.h>
#include <AvroPackets/s22.h>
#include <AvroPackets/hk101.h>
#include <AvroPackets/var102.h>
#include <AvroPackets/var103.h>
#include <AvroPackets/cmd151.h>
#include <Configurators/ConsumerConfigurator.h>
#include "Encoder.hh"
#include "Decoder.hh"
#include "ValidSchema.hh"
#include "Compiler.hh"
#include "DataFile.hh"
#include <cppkafka/configuration.h>
#include <cppkafka/cppkafka.h>

bool running = true;

int main(int argc, char *argv[]) {

	ConsumerConfigurator consConf(argc, argv);
	std::cout << "Starting cosumer with params: " << std::endl;
	std::cout << consConf.toString() << std::endl;

	// Stop processing on SIGINT
	signal(SIGINT, [](int) {
		running = false;
		std::cout << "Terminated by CTRL-C " << std::endl;
	});

	// Construct the configuration
	cppkafka::Configuration config = { { "metadata.broker.list",
			consConf.getKafkaIP().append(":").append(consConf.getKafkaPort()) },
			{ "group.id", consConf.getGroupID() },
			{ "auto.offset.reset" , "earliest"},
			//{ "enable.auto.offset.store" , false},
			{ "enable.auto.commit", true } };

	// Create the consumer
	cppkafka::Consumer consumer(config);

	// Subscribe to the topic
	std::string topic = consConf.getKafkaTopic();

	// Print the assigned partitions on assignment
	std::chrono::time_point start = std::chrono::steady_clock::now();
	consumer.set_assignment_callback(
			[&start](const cppkafka::TopicPartitionList &partitions) {
				std::cout << "Got assigned: " << partitions << std::endl;
				start = std::chrono::steady_clock::now();
			});
	// Print the revoked partitions on revocation

	consumer.set_revocation_callback(
			[](const cppkafka::TopicPartitionList &partitions) {
				std::cout << "Got revoked: " << partitions << std::endl;
			});

	//consumer.assign();
	consumer.subscribe( { topic });
	std::cout << "Consuming messages from topic " << topic << std::endl;

	// Now read messages from topic
	std::cout << "running..." << std::endl;

	int count = 0; // numero di pacchetti elaborati
	cppkafka::Message msg;
	while (running) {
		// Try to consume a message
		msg = consumer.poll();

		if (msg) {
			// If we managed to get a message
			if (msg.get_error()) {
				if (!msg.is_eof()) {
					std::cout << "[+] Received error notification: "
							<< msg.get_error() << std::endl;
				}

			} else {
				count += 1;

				//AVRO DECONDE INIT
				std::unique_ptr<avro::InputStream> in = avro::memoryInputStream(
						(const uint8_t*) msg.get_payload().get_data(),
						msg.get_payload().get_size());

				avro::DecoderPtr d = avro::binaryDecoder();
				d->init(*in);
				std::string key("null");
				try {
					key = std::string(msg.get_key());
					//std::cout << key << std::endl;
					if (key.compare("22") == 0) {
						S22::S22 s22;
						avro::decode(*d, s22);
					}
					else if (key.compare("11") == 0) {
						C11::C11 c11;
						avro::decode(*d, c11);
					}
					else if (key.compare("14") == 0) {
						C14::C14 c14;
						avro::decode(*d, c14);
					}
					else if (key.compare("101") == 0) {
						HK101::HK101 hk101;
						avro::decode(*d, hk101);

					}
					else if (key.compare("102") == 0) {
						VAR102::VAR102 var102;
						avro::decode(*d, var102);
					}
					else if (key.compare("103") == 0) {
						VAR103::VAR103 var103;
						avro::decode(*d, var103);
					}
					else if (key.compare("151") == 0) {
						CMD151::CMD151 cmd151;
						avro::decode(*d, cmd151);
					} else {
						std::cout << "key not recognized: " << key << std::endl;
						continue;
					}
				} catch (avro::Exception &e) {
					
					std::cout << key << " : " << e.what() << std::endl;
					continue;
				}

				if (count % 5000 == 0) {
					std::cout << "Tot Packets : " << count << std::endl;
					consumer.commit(msg);
				}

			}
		} 
	}
	if (msg)
		consumer.commit(msg);

	auto end = std::chrono::steady_clock::now(); //end timer

	unsigned int elps = std::chrono::duration_cast < std::chrono::microseconds
			> (end - start).count();

	std::cout << "Elapsed time in seconds: " << elps / pow(10, 6) << " s"
			<< std::endl;
	std::cout << "Tot Packets : " << count << std::endl;
	std::cout << "Packets to second : " << (count / (elps / pow(10, 6)))
			<< std::endl;
	consumer.unsubscribe();

}
